﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;

namespace KnightTour
{
    public partial class ChessBoard : UserControl
    {
        public event EventHandler<EventArgs> AnimationDone;

        public int DelayTime
        {
            get { return delayTime; }
            set
            {
                if (delayTime <= 0)
                {
                    throw new ArgumentOutOfRangeException();
                }
                delayTime = value;
            }
        }

        public bool IsBusy
        {
            get { return backgroundWorker.IsBusy; }
        }

        public ChessBoard()
        {
            InitializeComponent();

            CreateBoard();
        }

        public void Clear()
        {
            if (backgroundWorker.IsBusy)
            {
                throw new ApplicationException("Board is busy.");
            }

            for (int row = 0; row < 8; row++)
            {
                for (int col = 0; col < 8; col++)
                {
                    chessSquares[row][col].IsMarked = false;
                    chessSquares[row][col].ShowKnight = false;
                }
            }
            return;
        }

        public void ShowSolutionAnimation(int[][] board)
        {
            // Check argument(s).
            if (board == null)
            {
                throw new ArgumentNullException("board");
            }
            if (board.GetLength(0) != 8)
            {
                throw new ArgumentException("Invalid board.");
            }
            for (int i = 0; i < 8; i++)
            {
                if (board[i] == null)
                {
                    throw new ArgumentNullException(string.Format("board[{0}]", i));
                }
                if (board[i].GetLength(0) != 8)
                {
                    throw new ArgumentException("Invalid board.");
                }
            }
            for (int num = 1; num <= 64; num++)
            {
                bool found = false;
                for (int row = 0; row < 8; row++)
                {
                    for (int col = 0; col < 8; col++)
                    {
                        if (board[row][col] == num)
                        {
                            found = true;
                        }
                    }
                }
                if (!found)
                {
                    throw new ArgumentException("Invalid board.");
                }
            }

            if (backgroundWorker.IsBusy)
            {
                throw new ApplicationException("Board is busy.");
            }
            Clear();
            backgroundWorker.RunWorkerAsync(board);
            return;
        }

        public void StopSolutionAnimation()
        {
            if (backgroundWorker.IsBusy)
            {
                backgroundWorker.CancelAsync();
            }
            return;
        }

        private ChessSquare[][] chessSquares;
        private int delayTime = 1000;

        private void CreateBoard()
        {
            chessSquares = new ChessSquare[8][];

            SuspendLayout();
            for (int row = 0; row < 8; row++)
            {
                chessSquares[row] = new ChessSquare[8];
                for (int col = 0; col < 8; col++)
                {
                    ChessSquare chessSquare = new ChessSquare();
                    chessSquare.Size = new Size(48, 48);
                    chessSquare.Location = new Point(col * chessSquare.Size.Width, row * chessSquare.Size.Height);
                    if ((row + col) % 2 == 0)
                    {
                        chessSquare.IsWhiteHome = true;
                    }
                    else
                    {
                        chessSquare.IsWhiteHome = false;
                    }
                    chessSquare.IsMarked = false;
                    chessSquare.ShowKnight = false;

                    chessSquares[row][col] = chessSquare;
                    Controls.Add(chessSquare);
                }
            }
            ResumeLayout();
            return;
        }

        private delegate void SetChessSquareCallback(int row, int col, bool isMarked, bool showKnight);

        private void SetChessSquare(int row, int col, bool isMarked, bool showKnight)
        {
            if (InvokeRequired)
            {
                SetChessSquareCallback setChessSquareCallback = new SetChessSquareCallback(SetChessSquare);
                Invoke(setChessSquareCallback, row, col, isMarked, showKnight);
            }
            else
            {
                chessSquares[row][col].IsMarked = isMarked;
                chessSquares[row][col].ShowKnight = showKnight;
            }
            return;
        }

        private void backgroundWorker_DoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            int[][] board = (int[][])e.Argument;

            int previousRow = -1;
            int previousCol = -1;
            for (int num = 1; num <= 64; num++)
            {
                int row = -1;
                int col = -1;
                for (int i = 0; i < 8; i++)
                {
                    for (int j = 0; j < 8; j++)
                    {
                        if (board[i][j] == num)
                        {
                            row = i;
                            col = j;
                        }
                    }
                }

                if ((previousRow >= 0) && (previousCol >= 0))
                {
                    SetChessSquare(previousRow, previousCol, true, false);
                }

                if ((row >= 0) && (col >= 0))
                {
                    SetChessSquare(row, col, true, true);
                }

                previousRow = row;
                previousCol = col;

                // Wait.
                for (int i = 0; i < 1000; i++)
                {
                    Thread.Sleep(1);
                    if (backgroundWorker.CancellationPending)
                    {
                        return;
                    }
                }
            }
            return;
        }

        private void backgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            if (AnimationDone != null)
            {
                AnimationDone(this, EventArgs.Empty);
            }
            return;
        }
    }
}
